## Examples
## These are the base examples for testing, demonstrating, and teaching
## Media Computation using JES
##
## Started: Mark Guzdial, 2 July 2002

from media import *

#
# AUDIO EXAMPLES
#

# Simplest audio examples
def example1():
	f = '/Users/guzdial/mediasources/Elliot-hello.wav'
	s = makeSound(f)
	play(s)

def basicSound():
	play(sound(pickafile()))

def playparam(f):
	s = sound(f)
	play(s)

def playdouble(s):
	playAtRate(s,2)

# Manipulating samples
def louder(sound):
	for s in samples(sound):
		setSample(s,2*getSample(s))

def softer(sound):
	for s in samples(sound):
		setSample(s,0.5*getSample(s))

# Changing a sound along a function
def gettingLouder(sound):
	len = length(sound)
	counter = 0
	for s in samples(sound):
		counter += 4
		setSample(s, (counter/len)*getSample(s))

def gettingSofter(sound):
	len = length(sound)
	counter = 4 * len
	for s in samples(sound):
		counter -= 4
		setSample(s, (counter/len)*getSample(s))

#Play a sound at a recording
def samplePlayMain():
	fs = "/Users/guzdial/mediasources/croak.wav"
	s = sound(fs)
	play(s)		# Just to hear the original
	for f in [1046.5, 1174.6, 1318.5, 1396.9, 1567.9]:
		print "Playing at frequency "+str(f)
		playBitAtFreq(s,f)

def playBitAtFreq(sound,freq):
	rate = length(sound) * (freq / samplingRate(sound))
	sound.blockingPlayAtRateDur(rate,4000)  #Number of samples to play
	
def sampleDifferentMain():
	fs = "/Users/guzdial/mediasources/croak.wav"
	s = sound(fs)
	play(s)		# Just to hear the original
	playBitAtRate(s,2.0)
	for r in [0.5, 1.0, 1.5, 2.0, 2.5, 3.0]:
		print "Playing at rate "+str(r)
		playBitAtRate(s,r)

def playBitAtRate(sound,rate):
	sound.blockingPlayAtRateDur(rate,8000)

#Segmenting
# Here's what I got from exploring with the editor
# Word		Endpoint
# We		15730
# the		17407
# People	26726
# of		32131
# the		33413
# United	40052
# States	55510

def segmentMain():
	# Grab the sound JUST up to "We the people of the United States"
	fs="/Users/guzdial/mediasources/preamble.wav"
	s=sound(fs)
	nfs = "/Users/guzdial/mediasources/sec1silence.wav"  #An empty sound
	ns=sound(nfs)   # This is where we'll build the new sound
	nsi=1 # New sound index, starting from 1
	for si in range(1,55510):	# Where the samples are in the sound
		setSampleAt(ns,nsi, getSampleAt(s,si))
		nsi = nsi + 1
	play(ns)
	writeSoundTo(ns,"preamble-start.wav")
	
# Splicing
# Using the preamble piece, making "We the united people"
def spliceMain():
	fs="/Users/guzdial/mediasources/preamble-start.wav"
	s=sound(fs)
	ns=sound(fs)   # This is where we'll build the new sound
	nsi=17408     # New Sound Index starts at just after "We the" in the new sound
	for si in range(33414,40052):  # Where the word "United" is in the sound
		setSampleAt(ns,nsi, getSampleAt(s,si))
		nsi = nsi + 1
	for si in range(17408, 26726): # Where the word "People" is in the sound
		setSampleAt(ns,nsi, getSampleAt(s,si))
		nsi = nsi + 1
	for index in range(1,1000):		#Stick some empty space after that
		setSampleAt(ns,nsi,0)
		nsi = nsi + 1
	play(ns)	#Let's here and return the result
	return ns

# Not a real reverb yet, but this does create a definite "echo"
# A value of 5000 is about nice
def echo(filename,gap):
	s1 = sound(filename)
	smpl1 = samples(s1)
	s2 = sound(filename)
	smpl2 = samples(s2)
	for p in range(gap+1,length(s1)):
		s1.setSample(p, s1.getSample(p)+s2.getSample(p-gap))
	s1.play()

#Concatenating two sounds
# This isn't written to be particularly clear, but it sure is cool that it works!
def concat():
	fe='/Users/guzdial/mediasources/Elliot-hello.wav'
	se=sound(fe)
	fc4='/Users/guzdial/mediasources/c4.wav'
	sc4=sound(fc4)
	le=se.length()
	#Testing setting values
	se.setSample(le+1,0)
	se.setSample(le+2,0)
	for i in range(1,sc4.length()):
	    se.setSample(le+i,sc4.getSample(i))
	se.play()
	return se

#
# PICTURE EXAMPLES
#
def picexample1():
	f = "/Users/guzdial/mediasources/barbara.jpg"
	p = picture(f)
	show(p)

def decreaseRed(pic):
	for x in pixels(pic):
	   setRed(x, 0.95 * getRed(x))

def darken(pic):
	for x in pixels(pic):
		setColor(x,makeDarker(getColor(x)))
	
def lighten(pic):
	for p in pixels(pic):
		setColor(p,makeLighter(getColor(p)))

#
# Try titling
#
def testTitle():
	f = "/Users/guzdial/mediasources/barbara.jpg"
	newpic = title(f,"Copyright 2001 Mark Guzdial")	
	writePictureTo(newpic,"/Users/guzdial/mediasources/t-barbara.jpg")

def title(fname,string):
	p = picture(fname)
	drawText(p,30,30,string)
	show(p)
	return p


#
# Try titling a whole directory
#
def testTitling():
	import os.path
	os.path.walk("/Users/guzdial/mediasources/kid-in-bg-seq/",titleEach,"")

def titleEach(arg,dir,files):
	for f in files:
		newpic = title(dir+f,"Copyright 2001 Mark Guzdial")
		writePictureTo(newpic,dir+"t-"+f)
		
def swapBgMain():
	pic1 = "/Users/guzdial/mediasources/kid-in-frame.jpg"
	bg = "/Users/guzdial/mediasources/bgframe.jpg"
	newbg = "/Users/guzdial/mediasources/moon-surface.jpg"
	ppic1 = picture(pic1)
	pbg = picture(bg)
	pnewbg = picture(newbg)
	newp1 = swapbg(ppic1,pbg,pnewbg)
	show(newp1)
	show(pbg)
	show(pnewbg)
	return newp1

def swapbg(pic1, bg, newbg):
	for x in range(1,pic1.getWidth()):
		for y in range(1,pic1.getHeight()):
			p1px = getPixel(pic1,x,y)
			bgpx = getPixel(bg,x,y)
			if (distance(getColor(p1px),getColor(bgpx)) < 10.0):
				setColor(p1px,getColor(getPixel(newbg,x,y)))
	return pic1

def bluescreenMain():
	pic1 = "/Users/guzdial/mediasources/blue-mark.jpg"
	newbg = "/Users/guzdial/mediasources/moon-surface.jpg"
	ppic1 = picture(pic1)
	pnewbg = picture(newbg)
	newp1 = chromakey2(ppic1,pnewbg)
	show(newp1)
	return newp1

def chromakey(source,bg):
	# source should have something in front of blue, bg is the new background
	for x in range(1,source.getWidth()):
		for y in range(1,source.getHeight()):
			p = getPixel(source,x,y)
			if (getRed(p) + getGreen(p) < getBlue(p)):
				setColor(p,getColor(getPixel(bg,x,y)))
	return source

def chromakey2(source,bg):
	for p in pixels(source):
		if (getRed(p)+getGreen(p) < getBlue(p)):
			setColor(p,getColor(getPixel(bg,x(p),y(p))))
	return source